@@ -1,4 +1,7 @@
-#' EGRETci package for confidence interval analysis
+#' EGRETci package for bootstrap hypothesis tests and confidence interval analysis for WRTDS (Weighted 
+#' Regressions on Time, Discharge, and Season) statistical models. This 
+#' package is designed to be used in conjunction with the EGRET package, 
+#' which estimates and describes WRTDS models.
 #'
 #' \tabular{ll}{
 #' Package: \tab EGRETci\cr
@@ -18,12 +21,13 @@
 #' @name EGRETci-package
 #' @docType package
 #' @author Robert M. Hirsch \email{rhirsch@@usgs.gov}, Laura De Cicco \email{ldecicco@@usgs.gov}
-#' @references Hirsch, R.M., and De Cicco, L.A., 2014, User guide to Exploration and Graphics for RivEr Trends 
+#' @references Hirsch, R.M., and De Cicco, L.A., 2015, User guide to Exploration and Graphics for RivEr Trends 
 #' (EGRET) and dataRetrieval: R packages for hydrologic data: U.S. Geological Survey Techniques and Methods book 4, 
 #' chap. A10, 94 p., \url{http://dx.doi.org/10.3133/tm4A10}
 #' @references Hirsch, R.M., Archfield, S.A., and De Cicco, L.A., 2015, 
 #' A bootstrap method for estimating uncertainty of water quality trends.  
-#' Accepted for publication: Journal of Environmental Modelling and Software.
+#' Environmental Modelling & Software, 73, 148-166. 
+#' \url{https://www.sciencedirect.com/science/article/pii/S1364815215300220}
 #' @keywords water-quality graphics streamflow statistics 
 NULL
 
@@ -37,7 +41,7 @@ NULL
 #' @param eList named list with at least the Daily, Sample, and INFO dataframes. Created from the EGRET package, after running \code{\link[EGRET]{modelEstimation}}.
 #' @param \dots  additional arguments to bring in to reduce interactive options 
 #' (year1, year2, nBoot, bootBreak, blockLength)
-#' @keywords WRTDS flow
+#' @keywords WRTDS, water quality
 #' @return caseSetUp data frame with columns year1, yearData1, year2, yearData2, 
 #' numSamples, nBoot, bootBreak, blockLength, confStop. These correspond to:
 #' \tabular{ll}{
@@ -97,7 +101,7 @@ trendSetUp <- function(eList, ...){
   if(!is.null(matchReturn$bootBreak)){
     bootBreak <- as.numeric(matchReturn$bootBreak)
   } else {
-    message("Enter Mmin (minimum number of replicates), between 9 and nBoot, values of 39 or greater produce more accurate CIs")
+    message("Enter min (minimum number of replicates), between 9 and nBoot, values of 39 or greater produce more accurate CIs")
     bootBreak <- as.numeric(readline())
   }
 
@@ -114,6 +118,14 @@ trendSetUp <- function(eList, ...){
   message("blockLength = ",blockLength," this is the number of days in a bootstrap block")
 
   confStop <- 0.7
+  
+  if(year1 < floor(min(eList$Sample$DecYear, na.rm = TRUE))){
+    stop("year1 is less than the first Sample year")
+  }
+  
+  if(year2 > ceiling(max(eList$Sample$DecYear, na.rm = TRUE))){
+    stop("year2 is greater than the last Sample year")
+  }
 
   caseSetUp <- data.frame(year1=year1,
                           yearData1=yearData1,
@@ -129,7 +141,8 @@ trendSetUp <- function(eList, ...){
   
 }
 
-#' Save EGRETci workspace after wBT
+#' Save EGRETci workspace after running wBT (the WRTDS bootstrap test)
+#'
 #'
 #' Saves critical information in a EGRETci workflow when analyzing trends over a set of two years.
 #'
@@ -165,28 +178,38 @@ saveEGRETci <- function(eList, eBoot, caseSetUp, fileName = ""){
 #' confidence intervals for the trends between two specified years.  The trends 
 #' evaluated are trends in flow normalized concentration and flow normalized flux.  
 #' Function produces text outputs and a named list (eBoot) that contains all of the 
-#' relevant outputs.
+#' relevant outputs. Check out \code{\link{runPairsBoot}} and \code{\link{runGroupsBoot}}
+#' for more bootstrapping options.
 #'
 #' @param eList named list with at least the Daily, Sample, and INFO dataframes. Created from the EGRET package, after running \code{\link[EGRET]{modelEstimation}}.
 #' @param caseSetUp data frame. Returned from \code{\link{trendSetUp}}.
 #' @param saveOutput logical. If \code{TRUE}, a text file will be saved in the working directory.
 #' @param fileName character. Name to save the output file if \code{saveOutput=TRUE}.
-#' @return eBoot, a named list with bootOut,wordsOut,xConc,xFlux values
-#' @importFrom EGRET as.egret
+#' @param startSeed setSeed value. Defaults to 494817. This is used to make repeatable output.
 #' @importFrom binom binom.bayes
 #' @importFrom stats quantile
 #' @export
-#' @seealso \code{\link{trendSetUp}}, \code{\link{setForBoot}}
+#' @return eBoot, a named list with bootOut,wordsOut,xConc,xFlux values. bootOut is a data frame with the results
+#' of the bootstrapping tests. wordsOut is a character vector describing the results.
+#' xConc, xFlux are vectors of length iBoot, of the change in flow normalized concentration or flux 
+#' computed by each bootstrap replicate (mg/L). pConc and pFlux are vectors of length iBoot, of the change 
+#' in flow normalized concentration or flux computed from each bootstrap replicate expressed as % change.
+#' @seealso \code{\link{trendSetUp}}, \code{\link{setForBoot}}, \code{\link{runGroupsBoot}}, \code{\link{runPairsBoot}}
 #' @examples
 #' library(EGRET)
 #' eList <- Choptank_eList
 #' \dontrun{
-#' caseSetUp <- trendSetUp(eList)
+#'   caseSetUp <- trendSetUp(eList,
+#'   year1=1985, 
+#'   year2=2005,
+#'   nBoot = 50, 
+#'   bootBreak = 39,
+#'   blockLength = 200)
 #' eBoot <- wBT(eList,caseSetUp)
 #' }
 wBT<-function(eList,caseSetUp, 
-              saveOutput=TRUE, fileName="temp.txt"){
-
+              saveOutput=TRUE, fileName="temp.txt", startSeed = 494817){
+  
   #   This is the version of wBT that includes the revised calculation of the 
   #    two-sided p-value, added 16Jul2015, RMHirsch
   #
@@ -215,24 +238,25 @@ wBT<-function(eList,caseSetUp,
   nBoot <- caseSetUp$nBoot
   bootBreak <- caseSetUp$bootBreak
   blockLength <- caseSetUp$blockLength
-  periodName <- setSeasonLabel(data.frame(PeriodStart = localINFO$paStart, 
+  periodName <- EGRET::setSeasonLabel(data.frame(PeriodStart = localINFO$paStart, 
                                           PeriodLong = localINFO$paLong))
   confStop <- caseSetUp$confStop
+  
   xConc <- rep(NA, nBoot)
   xFlux <- rep(NA, nBoot)
   pConc <- rep(NA, nBoot)
   pFlux <- rep(NA, nBoot)
   posXConc <- 0
   posXFlux <- 0
-  possibleError1 <- tryCatch(surfaces1 <- estSliceSurfacesSimpleAlt(eList, 
-                                                                    year1), error = function(e) e)
-  possibleError2 <- tryCatch(surfaces2 <- estSliceSurfacesSimpleAlt(eList, 
-                                                                    year2), error = function(e) e)
-  if (!inherits(possibleError1, "error") & !inherits(possibleError2, 
-                                                     "error")) {
+  possibleError1 <- tryCatch(surfaces1 <- estSliceSurfacesSimpleAlt(eList, year1), error = function(e) e)
+  possibleError2 <- tryCatch(surfaces2 <- estSliceSurfacesSimpleAlt(eList, year2), error = function(e) e)
+  
+  if (!inherits(possibleError1, "error") &&
+      !inherits(possibleError2, "error")) {
+    
     combo <- makeCombo(surfaces1, surfaces2)
-    eListCombo <- as.egret(localINFO, localDaily, localSample, 
-                           combo)
+    eListCombo <- suppressMessages(EGRET::as.egret(localINFO, localDaily, localSample, 
+                           combo))
     res <- makeTwoYearsResults(eListCombo, year1, year2)
     regDeltaConc <- res[2] - res[1]
     estC <- regDeltaConc
@@ -251,8 +275,17 @@ wBT<-function(eList,caseSetUp,
     }
     cat("\n\n", eList$INFO$shortName, "  ", eList$INFO$paramShortName)
     cat("\n\n", periodName)
-    cat("\n\n  Bootstrap process, for change from Water Year", 
-        year1, "to Water Year", year2)
+    if(eList$INFO$paStart == 1 & eList$INFO$paLong == 12){
+      cat("\n\n  Bootstrap process, for change from Calendar Year", 
+          year1, " to ", year2)      
+    } else if (eList$INFO$paStart == 10 & eList$INFO$paLong == 12){
+      cat("\n\n  Bootstrap process, for change from Water Year ", 
+          year1, " to Water Year ", year2)       
+    } else {
+      cat("\n\n  Bootstrap process, for change from ", 
+          year1, " to ", year2, ":",periodName) 
+    }
+    
     cat("\n                   data set runs from Water Year", 
         yearData1, "to Water Year", yearData2)
     cat("\n  Bootstrap block length in days", blockLength)
@@ -261,14 +294,22 @@ wBT<-function(eList,caseSetUp,
         " mg/L")
     cat("\n WRTDS estimated flux change is        ", ffc, 
         " 10^6 kg/yr")
-
+    
     if (saveOutput) {
       message("\n", eList$INFO$shortName, "  ", eList$INFO$paramShortName)
       message("\n", periodName)
-      message("\n  Bootstrap process, for change from Water Year ", 
-              year1, " to Water Year ", year2)
-      message("                   data set runs from WaterYear ", 
-              yearData1, " to Water Year ", yearData2)
+      if(eList$INFO$paStart == 1 & eList$INFO$paLong == 12){
+        message("\n  Bootstrap process, for change from Calendar Year", 
+                year1, " to ", year2)      
+      } else if (eList$INFO$paStart == 10 & eList$INFO$paLong == 12){
+        message("\n  Bootstrap process, for change from Water Year", 
+                year1, " to Water Year", year2)       
+      } else {
+        message("\n  Bootstrap process, for change from ", 
+                year1, " to ", year2, ":",periodName) 
+      }
+      message("                   data set runs from ", 
+              yearData1, " to Year ", yearData2)
       message("  Bootstrap block length in days ", blockLength)
       message("  bootBreak is ", bootBreak, "  confStop is ", 
               confStop)
@@ -282,11 +323,13 @@ wBT<-function(eList,caseSetUp,
       message("\n      rep              Concentration             |              Flux")
       message("          value     nPos post_p   Lower   Upper  |     value   nPos  post_p    Lower   Upper")
     }
-    for (iBoot in 1:nBoot) {
+    nBootGood <- 0
+    for (iBoot in 1:(2*nBoot)) {
+
       bootSample <- blockSample(localSample = localSample, 
-                                blockLength = blockLength)
-      eListBoot <- as.egret(localINFO, localDaily, bootSample, 
-                            NA)
+                                blockLength = blockLength,
+                                startSeed = startSeed + iBoot)
+      eListBoot <- suppressMessages(EGRET::as.egret(localINFO, localDaily, bootSample, NA))
       possibleError3 <- tryCatch(surfaces1 <- estSliceSurfacesSimpleAlt(eListBoot, 
                                                                         year1), error = function(e) e)
       possibleError4 <- tryCatch(surfaces2 <- estSliceSurfacesSimpleAlt(eListBoot, 
@@ -294,47 +337,46 @@ wBT<-function(eList,caseSetUp,
       if (!inherits(possibleError3, "error") & !inherits(possibleError4, 
                                                          "error")) {
         combo <- makeCombo(surfaces1, surfaces2)
-        eListBoot <- as.egret(localINFO, localDaily, 
-                              bootSample, combo)
-        res <- makeTwoYearsResults(eListBoot, year1, 
-                                   year2)
-        xConc[iBoot] <- (2 * regDeltaConc) - (res[2] - 
-                                                res[1])
-        xFlux[iBoot] <- (2 * regDeltaFlux) - ((res[4] - 
-                                                 res[3]) * 0.00036525)
+        eListBoot <- suppressMessages(EGRET::as.egret(localINFO, localDaily, bootSample, combo))
+        res <- makeTwoYearsResults(eListBoot, year1, year2)
+        
+        nBootGood <- nBootGood + 1
+        
+        xConc[nBootGood] <- (2 * regDeltaConc) - (res[2] - res[1])
+        xFlux[nBootGood] <- (2 * regDeltaFlux) - ((res[4] - res[3]) * 0.00036525)
         LConc <- (2 * LConcDiff) - (log(res[2]) - log(res[1]))
-        pConc[iBoot] <- (100 * exp(LConc)) - 100
+        pConc[nBootGood] <- (100 * exp(LConc)) - 100
         LFlux <- (2 * LFluxDiff) - (log(res[4]) - log(res[3]))
-        pFlux[iBoot] <- (100 * exp(LFlux)) - 100
-        posXConc <- ifelse(xConc[iBoot] > 0, posXConc + 
-                             1, posXConc)
-        binomIntConc <- binom::binom.bayes(posXConc, 
-                                           iBoot, confStop, "central")
+        pFlux[nBootGood] <- (100 * exp(LFlux)) - 100
+        ####### From here on out, no longer parallizable:
+        posXConc <- ifelse(xConc[nBootGood] > 0, posXConc + 1, posXConc)
+        binomIntConc <- binom::binom.bayes(posXConc, nBootGood, confStop, "central")
         belowConc <- ifelse(binomIntConc$upper < 0.05, 1, 0)
         aboveConc <- ifelse(binomIntConc$lower > 0.95, 1, 0)
         midConc <- ifelse(binomIntConc$lower > 0.05 & 
                             binomIntConc$upper < 0.95, 1, 0)
-        posXFlux <- ifelse(xFlux[iBoot] > 0, posXFlux + 
-                             1, posXFlux)
+        posXFlux <- ifelse(xFlux[nBootGood] > 0, posXFlux + 1, posXFlux)
         binomIntFlux <- binom::binom.bayes(posXFlux, 
                                            iBoot, confStop, "central")
         belowFlux <- ifelse(binomIntFlux$upper < 0.05, 1, 0)
         aboveFlux <- ifelse(binomIntFlux$lower > 0.95, 1, 0)
         midFlux <- ifelse(binomIntFlux$lower > 0.05 & 
                             binomIntFlux$upper < 0.95, 1, 0)
-        quantConc <- quantile(xConc[1:iBoot], prob, type = 6)
+        
+        quantConc <- quantile(xConc[1:nBootGood], prob, type = 6, na.rm = TRUE)
         lowConc <- quantConc[2]
         highConc <- quantConc[8]
-        quantFlux <- quantile(xFlux[1:iBoot], prob, type = 6)
+        quantFlux <- quantile(xFlux[1:nBootGood], prob, type = 6, na.rm = TRUE)
         lowFlux <- quantFlux[2]
         highFlux <- quantFlux[8]
-        prints <- c(format(iBoot, digits = 3, width = 7), 
-                    format(xConc[iBoot], digits = 3, width = 7), 
+        
+        prints <- c(format(nBootGood, digits = 3, width = 7), 
+                    format(xConc[nBootGood], digits = 3, width = 7), 
                     format(posXConc, digits = 3, width = 5), 
                     format(binomIntConc$mean,digits = 3), 
                     format(quantConc[2], digits = 3, width = 7), 
                     format(quantConc[8], digits = 3, width = 7), "  |  ", 
-                    format(xFlux[iBoot], digits = 4, width = 8), 
+                    format(xFlux[nBootGood], digits = 4, width = 8), 
                     format(posXFlux, digits = 3, width = 5), 
                     format(binomIntFlux$mean, digits = 3, width = 7), 
                     format(quantFlux[2], digits = 4, width = 8), 
@@ -353,20 +395,24 @@ wBT<-function(eList,caseSetUp,
         
         test1 <- as.numeric(belowConc + aboveConc + midConc > 
                               0.5 & belowFlux + aboveFlux + midFlux > 0.5 & 
-                              iBoot >= bootBreak & iBoot > 30)
+                              nBootGood >= bootBreak & iBoot > 30)
         test2 <- as.numeric(midConc > 0.5 & midFlux > 
-                              0.5 & iBoot >= bootBreak & iBoot <= 30)
-        if (test1 + test2 > 0.5) {
+                              0.5 & nBootGood >= bootBreak & nBootGood <= 30)
+        if (!is.na(test1) && !is.na(test2) && test1 + test2 > 0.5) {
           break
         }
-      }
-      else {
-        if (saveOutput) {
-          sink()
+        if(nBootGood >= nBoot) {
+          break()
         }
-        stop(possibleError3, "/n", possibleError4)
-      }
+      } 
     }
+    
+    if(iBoot == 2*nBoot){
+      message(iBoot, " iterations were run. They only achieved ", nBootGood, " sucessful runs.")
+    } else if (iBoot > nBoot){
+      message("It took ", iBoot, " iterations to achieve ", nBoot, " sucessful runs.")
+    }
+
     rejectC <- lowConc * highConc > 0
     rejectF <- lowFlux * highFlux > 0
     cat("\n\nShould we reject Ho that Flow Normalized Concentration Trend = 0 ?", 
@@ -385,9 +431,10 @@ wBT<-function(eList,caseSetUp,
     pValC <- pVal(xConc)
     cat("\n approximate two-sided p-value for Conc", format(pValC, 
                                                             digits = 2, width = 9))
-    if (posXConc == 0 | posXConc == iBoot) 
+    if (!is.na(posXConc) && ( posXConc == 0 | posXConc == nBootGood) ){
       cat("\n* Note p-value should be considered to be < stated value")
-    likeCUp <- (posXConc + 0.5)/(iBoot + 1)
+    }
+    likeCUp <- (posXConc + 0.5)/(nBootGood + 1)
     likeCDown <- 1 - likeCUp
     cat("\n Likelihood that Flow Normalized Concentration is trending up =", 
         format(likeCUp, digits = 3, width = 10), " is trending down =", 
@@ -409,9 +456,11 @@ wBT<-function(eList,caseSetUp,
     pValF <- pVal(xFlux)
     cat("\n approximate two-sided p-value for Flux", format(pValF, 
                                                             digits = 2, width = 9))
-    if (posXFlux == 0 | posXFlux == iBoot) 
+    if (!is.na(posXFlux) && (posXFlux == 0 | posXFlux == nBootGood)) {
       cat("\n* Note p-value should be considered to be < stated value")
-    likeFUp <- (posXFlux + 0.5)/(iBoot + 1)
+    }
+    
+    likeFUp <- (posXFlux + 0.5)/(nBootGood + 1)
     likeFDown <- 1 - likeFUp
     cat("\n Likelihood that Flow Normalized Flux is trending up =", 
         format(likeFUp, digits = 3), " is trending down=", 
@@ -419,7 +468,8 @@ wBT<-function(eList,caseSetUp,
     bootOut <- data.frame(rejectC, pValC, estC, lowC, upC, 
                           lowC50, upC50, lowC95, upC95, likeCUp, likeCDown, 
                           rejectF, pValF, estF, lowF, upF, lowF50, upF50, lowF95, 
-                          upF95, likeFUp, likeFDown, baseConc, baseFlux, iBoot)
+                          upF95, likeFUp, likeFDown, baseConc, baseFlux, iBoot,
+                          startSeed, nBootGood)
     likeList <- c(likeCUp, likeCDown, likeFUp, likeFDown)
     wordsOut <- wordLike(likeList)
     cat("\n\n", format(wordsOut[1], width = 30), "\n", format(wordsOut[3], 
@@ -440,8 +490,11 @@ wBT<-function(eList,caseSetUp,
               fquantConc[2], " ", fquantConc[8])
       message("  also 95% CIs", fquantConc[1], " ", fquantConc[9], 
               "\n and 50% CIs ", fquantConc[4], " ", fquantConc[6])
-      if (posXConc == 0 | posXConc == iBoot) 
+      
+      if (!is.na(posXConc) && (posXConc == 0 | posXConc == iBoot)) {
         message("* Note p-value should be considered to be < stated value")
+      }
+      
       message("  approximate two-sided p-value for Conc ", 
               format(pValC, digits = 2, width = 9))
       message("  Likelihood that Flow Normalized Concentration is trending up = ", 
@@ -455,8 +508,11 @@ wBT<-function(eList,caseSetUp,
               "\n and 50% CIs ", fquantFlux[4], " ", fquantFlux[6])
       message("  approximate two-sided p-value for Flux ", 
               format(pValF, digits = 2, width = 9))
-      if (posXFlux == 0 | posXFlux == iBoot) 
+      
+      if (!is.na(posXFlux) && (posXFlux == 0 | posXFlux == iBoot)){
         message("* Note p-value should be considered to be < stated value")
+      } 
+      
       message("  Likelihood that Flow Normalized Flux is trending up = ", 
               format(likeFUp, digits = 3), " is trending down= ", 
               format(likeFDown, digits = 3))
@@ -465,33 +521,28 @@ wBT<-function(eList,caseSetUp,
       message(" ", format(wordsOut[2], width = 30), "\n ", 
               format(wordsOut[4], width = 30))
     }
+    attr(eBoot, "year1") <- year1
+    attr(eBoot, "year2") <- year2
     return(eBoot)
-  }
-  else {
-    stop(possibleError1, "/n", possibleError2)
+  } else {
+    if("message" %in% names(possibleError1) && 
+       "message" %in% names(possibleError2)){
+      stop(possibleError1$message, "/n", possibleError2)
+    } else if ("message" %in% names(possibleError1)){
+      stop(possibleError1$message)
+    } else {
+      stop(possibleError2$message)
+    }
+    
   }
 }
+  
 
-#' surface slice
-#'
-#' Creates surface slice for one year.
-#'
-#' @param eList named list with at least the Daily, Sample, and INFO dataframes. Created from the EGRET package, 
-#' after running either \code{\link[EGRET]{modelEstimation}} or \code{\link{setForBoot}}.
-#' @param year integer year to perform WRTDS analysis
-#' @keywords WRTDS flow
-#' @importFrom EGRET runSurvReg
-#' @return surfaces matrix
-#' @export
-#' @examples
-#' library(EGRET)
-#' eList <- Choptank_eList
-#' \dontrun{
-#' caseSetUp <- trendSetUp(eList, nBoot=100, blockLength=200)
-#' eList <- setForBoot(eList, caseSetUp)
-#' surfaces <- estSliceSurfacesSimpleAlt(eList, 1990)
-#' }
-estSliceSurfacesSimpleAlt<-function(eList,year){
+  
+
+
+estSliceSurfacesSimpleAlt <- function(eList,year){
+  
   localINFO <- eList$INFO
   localSample <- eList$Sample
   localDaily <- eList$Daily
@@ -508,126 +559,95 @@ estSliceSurfacesSimpleAlt<-function(eList,year){
   originalColumns <- names(localSample)
   minNumUncen <- min(c(localINFO$minNumUncen, sum(localSample$Uncen)), na.rm=TRUE)
   minNumObs <- min(c(localINFO$minNumObs, length(localSample$ConcLow)), na.rm=TRUE)
-  # minNumUncen <- min(c(localINFO$minNumUncen, 0.5), na.rm=TRUE)
+  
   bottomLogQ <- localINFO$bottomLogQ
   stepLogQ <- localINFO$stepLogQ
-  topLogQ<-bottomLogQ + 13 * stepLogQ
-  vectorLogQ<-seq(bottomLogQ,topLogQ,stepLogQ)
+  topLogQ <- bottomLogQ + 13 * stepLogQ
+  vectorLogQ <- seq(bottomLogQ,topLogQ,stepLogQ)
   nVectorLogQ <- localINFO$nVectorLogQ
-  stepYear<- localINFO$stepYear
-  bottomYear<-localINFO$bottomYear
+  stepYear <- localINFO$stepYear
+  bottomYear <-localINFO$bottomYear
   nVectorYear <- localINFO$nVectorYear
-  topYear<-bottomYear + (nVectorYear - 1)* stepYear 
+  topYear <- bottomYear + (nVectorYear - 1)* stepYear 
   vectorYear <-seq(bottomYear,topYear,stepYear)
-  surfaces<-array(NA,dim=c(14,length(vectorYear),3))
-  
-#   if(is.null(localINFO$paStart) | is.null(localINFO$paLong)){
-#     # Default to water year
-#     localINFO$paStart <- 10
-#     localINFO$paLong <- 12
-#   }
+  surfaces <- array(NA,dim=c(14,length(vectorYear),3))
   
   vectorIndex <- paVector(year,localINFO$paStart,localINFO$paLong,vectorYear)
   # Tack on one data point on either side
   vectorIndex <- c(vectorIndex[1]-1,vectorIndex,vectorIndex[length(vectorIndex)]+1)
+
+  vectorIndex <- vectorIndex[vectorIndex != 0]
   
   vectorYear <- vectorYear[vectorIndex]
-  nVectorYear<-length(vectorYear)
-  estPtLogQ<-rep(vectorLogQ,nVectorYear)
-  estPtYear<-rep(vectorYear,each=14)
+  nVectorYear <- length(vectorYear)
+  estPtLogQ <- rep(vectorLogQ,nVectorYear)
+  estPtYear <- rep(vectorYear,each=14)
   
   numDays <- localINFO$numDays
   DecLow <- localINFO$DecLow
   DecHigh <- localINFO$DecHigh
   
-  resultSurvReg <- runSurvReg(estPtYear,estPtLogQ,numDays,
-                                     DecLow,DecHigh, localSample,
-                                     windowY,windowQ,windowS,
-                                     minNumObs,minNumUncen,
-                                     interactive=FALSE,edgeAdjust)
+  if(utils::packageVersion("EGRET") >= "2.6.1"){
+    resultSurvReg <- EGRET::runSurvReg(estPtYear = estPtYear,estPtLQ = estPtLogQ,
+                                DecLow = DecLow,DecHigh = DecHigh, 
+                                Sample = localSample,windowY = windowY,windowQ = windowQ,
+                                windowS = windowS,minNumObs = minNumObs,minNumUncen = minNumUncen,
+                                verbose =FALSE,edgeAdjust = edgeAdjust)
+  } else {
+    message("Consider updating the EGRET package")
+    resultSurvReg <- EGRET::runSurvReg(estPtYear = estPtYear,estPtLQ = estPtLogQ,
+                                numDays = numDays,DecLow = DecLow,DecHigh = DecHigh, 
+                                Sample = localSample,windowY = windowY,windowQ = windowQ,
+                                windowS = windowS,minNumObs = minNumObs,minNumUncen = minNumUncen,
+                                interactive =  FALSE,edgeAdjust = edgeAdjust)
+  }
+
   
   for(iQ in 1:14) {
     for(iY in 1:length(vectorIndex)){ 
       k<-(iY-1)*14+iQ
-      surfaces[iQ,vectorIndex[iY],]<-resultSurvReg[k,]
+      if(k <= dim(resultSurvReg)[1]){
+        surfaces[iQ,vectorIndex[iY],]<-resultSurvReg[k,]
+      }
     }
   }
   
   return(surfaces)
 }
 
-#' paVector
-#'
-#' Creates paVector. This is the index of which years are in the proper period of record.
-#'
-#' @param year integer year to look for. If the period of analysis is a water 
-#' year (\code{setPA(paStart = 10, paLong = 12)}), the year corresponds to the calendar year of that water year for Jan-Sept.
-#' If the period of record crosses a calendar year (\code{setPA(paStart=10, paLong=3)}), the year indicates the year at the ending month.
-#' @param paStart integer starting month for period of analysis
-#' @param paLong integer length of period of analysis
-#' @param vectorYear numeric vector of decimal years
-#' @keywords WRTDS flow
-#' @import lubridate
-#' @return surfaces matrix
-#' @export
-#' @examples
-#' year <- 2000
-#' paStart <- 10
-#' paLong <- 12
-#' vectorYear <- c(seq(1999,2001,0.1))
-#' paIndexWaterYear <- paVector(year, paStart, paLong, vectorYear)
-#' requestedYears <- vectorYear[paIndexWaterYear]
-#' paStart <- 11
-#' paLong <- 3
-#' paIndexWinter <- paVector(year, paStart, paLong, vectorYear)
-#' requestedWinterYears <- vectorYear[paIndexWinter]
-#' paStart <- 6
-#' paLong <- 3
-#' paIndexSummer <- paVector(year, paStart, paLong, vectorYear)
-#' requestedSummerYears <- vectorYear[paIndexSummer]
-#' paStart <- 10
-#' paLong <- 3
-#' paIndexLate <- paVector(year, paStart, paLong, vectorYear)
-#' endOfYear <- vectorYear[paIndexLate]
 paVector <- function(year,paStart,paLong, vectorYear){
   
   if (paStart + paLong > 13){
     # Crosses January
-    minTime <- ymd(paste(year-1,paStart,1,sep="-"))
-    maxTime <- as.Date(ymd(paste(year,(paStart + paLong - 12),1)))-1
+    minYear_int <- year-1
+    maxTime <- as.POSIXct(as.Date(paste(year,(paStart + paLong - 12),1,sep="-"))-1)
   } else {
-    minTime <- ymd(paste(year,paStart,1,sep="-"))
+    minYear_int <- year
     if(paStart + paLong <= 12){
-      maxTime <- as.Date(ymd(paste(year,(paStart + paLong),1)))-1
+      maxTime <- as.POSIXct(as.Date(paste(year,(paStart + paLong),1,sep="-"))-1)
     } else {
       #Special december issue
-      maxTime <- as.Date(ymd(paste(year,12,31)))
+      maxTime <- as.POSIXct(as.Date(paste(year,12,31,sep="-")))
     }
-    
   }
   
-  minTime <- decimal_date(minTime)
+  minTime <- as.POSIXct(paste(minYear_int,paStart,1,sep="-"))
+  minYear <- as.POSIXct(paste0(minYear_int,"-01-01 00:00"))
+  endMinYear <- as.POSIXct(paste0(minYear_int,"-12-31 23:59"))
   
-  maxTime <- decimal_date(maxTime)
+  maxYear <- as.POSIXct(paste0(year,"-01-01 00:00"))
+  endMaxYear <- as.POSIXct(paste0(year,"-12-31 23:59"))
   
-  vectorIndex <- which(vectorYear >= minTime & vectorYear <= maxTime)
+  minTime_dec <- minYear_int + as.numeric(difftime(minTime, minYear, units = "secs"))/as.numeric(difftime(endMinYear, minYear, units = "secs"))
+
+  maxTime_dec <- year + as.numeric(difftime(maxTime, maxYear, units = "secs"))/as.numeric(difftime(endMaxYear, maxYear, units = "secs"))
+  
+  vectorIndex <- which(vectorYear >= minTime_dec & vectorYear <= maxTime_dec)
   
   return(vectorIndex)
 }
 
-#' makeCombo
-#'
-#' Combine surface slices.
-#'
-#' @param surfaces1 vector returned from \code{\link{estSliceSurfacesSimpleAlt}}
-#' @param surfaces2 vector returned from \code{\link{estSliceSurfacesSimpleAlt}}
-#' @keywords WRTDS flow
-#' @return surfaces matrix
-#' @export
-#' @examples
-#' surfaces1 <- c(1,2,3)
-#' surfaces2 <- c(4, NA, 5)
-#' surfaces <- makeCombo(surfaces1, surfaces2)
+
 makeCombo <- function (surfaces1,surfaces2) {
 	surfaces1[is.na(surfaces1)]<-0
 	surfaces2[is.na(surfaces2)]<-0
@@ -636,41 +656,18 @@ makeCombo <- function (surfaces1,surfaces2) {
 	return(combo)
 }
 
-#' makeTwoYearsResults
-#'
-#' In bootstrap process computes the results for the initial year and second year.
-#'
-#' @param eList named list with at least the Daily, Sample, and INFO dataframes. Created from the EGRET package, after running \code{\link[EGRET]{modelEstimation}}.
-#' @param year1 integer. Initial year of a 2-year trend comparison.
-#' @param year2 integer. Second year of a 2-year trend comparison.
-#' @keywords WRTDS flow
-#' @importFrom EGRET estDailyFromSurfaces
-#' @importFrom EGRET setupYears
-#' @return surfaces matrix
-#' @export
-#' @examples
-#' library(EGRET)
-#' eList <- Choptank_eList
-#' 
-#' twoResultsWaterYear <- makeTwoYearsResults(eList, 1985, 2005)
 makeTwoYearsResults <- function(eList,year1,year2){
 
   paStart <- eList$INFO$paStart
   paLong <- eList$INFO$paLong
-	returnDaily <- estDailyFromSurfaces(eList)
+	returnDaily <- EGRET::estDailyFromSurfaces(eList)
   
-	bootAnnRes<- setupYears(localDaily=returnDaily, 
+	bootAnnRes<- EGRET::setupYears(localDaily=returnDaily, 
                                  paStart=paStart, 
                                  paLong=paLong)
 	twoYearsResults <- c(bootAnnRes$FNConc[!is.na(bootAnnRes$FNConc)],
 	                     bootAnnRes$FNFlux[!is.na(bootAnnRes$FNFlux)])
-# 	AnnBase <- bootAnnRes[1,1]
-# 	index1 <- year1 - trunc(AnnBase) + 1
-# 	index2 <- year2 - trunc(AnnBase) + 1
-# 	twoYearsResults <- c(bootAnnRes$FNConc[index1],
-#                        bootAnnRes$FNConc[index2],
-#                        bootAnnRes$FNFlux[index1],
-#                        bootAnnRes$FNFlux[index2])
+
 	return(twoYearsResults)
 }
 
@@ -685,8 +682,7 @@ makeTwoYearsResults <- function(eList,year1,year2){
 #' @param windowS numeric specifying the half-window with in the seasonal dimension, in units of years, default is 0.5
 #' @param edgeAdjust logical specifying whether to use the modified method for calculating the windows at the edge of the record.  
 #' @keywords WRTDS flow
-#' @importFrom EGRET surfaceIndex
-#' @return surfaces matrix
+#' @return eList list with Daily,Sample, INFO data frames and surface matrix.
 #' @export
 #' @examples
 #' library(EGRET)
@@ -731,14 +727,22 @@ setForBoot<-function (eList,caseSetUp, windowY = 7, windowQ = 2,
 	  localINFO$minNumUncen <- min(100, numSamples - 20)
 	}
   
-  surfaceIndexParameters <- surfaceIndex(localDaily)
-  localINFO$bottomLogQ <- surfaceIndexParameters[1]
-  localINFO$stepLogQ <- surfaceIndexParameters[2]
-  localINFO$nVectorLogQ <- surfaceIndexParameters[3]
-  localINFO$bottomYear <- surfaceIndexParameters[4]
-  localINFO$stepYear <- surfaceIndexParameters[5]
-  localINFO$nVectorYear <- surfaceIndexParameters[6]
-
+  surfaceIndexParameters <- EGRET::surfaceIndex(localDaily)
+  if(utils::packageVersion("EGRET") > '2.6.1'){
+    localINFO$bottomLogQ <- surfaceIndexParameters[['bottomLogQ']]
+    localINFO$stepLogQ <- surfaceIndexParameters[['stepLogQ']]
+    localINFO$nVectorLogQ <- surfaceIndexParameters[['nVectorLogQ']]
+    localINFO$bottomYear <- surfaceIndexParameters[['bottomYear']]
+    localINFO$stepYear <- surfaceIndexParameters[['stepYear']]
+    localINFO$nVectorYear <- surfaceIndexParameters[['nVectorYear']]
+  } else {
+    localINFO$bottomLogQ <- surfaceIndexParameters[1]
+    localINFO$stepLogQ <- surfaceIndexParameters[2]
+    localINFO$nVectorLogQ <- surfaceIndexParameters[3]
+    localINFO$bottomYear <- surfaceIndexParameters[4]
+    localINFO$stepYear <- surfaceIndexParameters[5]
+    localINFO$nVectorYear <- surfaceIndexParameters[6]
+  }
 
   localINFO$numDays <- numDays
   localINFO$DecLow <- DecLow
@@ -751,19 +755,31 @@ setForBoot<-function (eList,caseSetUp, windowY = 7, windowQ = 2,
 
 #' blockSample
 #'
-#' Get a subset of the Sample data frame based on the user-specified blockLength.
+#' Get a random subset of the Sample data frame based on the user-specified blockLength 
+#' for use in bootstrap estimation process. The subset is a random subset of blocks of 
+#' data from Sample dataframe.  The subset is based on the random selection (with 
+#' replacement) of starting dates from the full Sample data frame.  The Sample selected 
+#' has the same number of observations as the original Sample (some 
+#' observations are are included once, some included multiple times, and some are not 
+#' included).
 #'
 #' @param localSample Sample data frame
-#' @param blockLength integer size of subset.
+#' @param blockLength integer size of subset expressed in days.
+#' @param startSeed setSeed value. Defaults to 494817. This is used to make repeatable output.
 #' @keywords WRTDS flow
-#' @return surfaces matrix
+#' @return newSample data frame in same format as Sample data frame
 #' @export
 #' @examples
 #' library(EGRET)
 #' eList <- Choptank_eList
 #' Sample <- eList$Sample
-#' bsReturn <- blockSample(Sample, 25)
-blockSample <- function(localSample, blockLength){
+#' bsReturn <- blockSample(Sample, 200)
+blockSample <- function(localSample, blockLength, startSeed = NA){
+  
+  if(!is.na(startSeed)){
+    set.seed(startSeed)
+  }
+  
   numSamples <- length(localSample$Julian)
   dayOne <- localSample$Julian[1]
   newSample <- data.frame()
@@ -774,8 +790,8 @@ blockSample <- function(localSample, blockLength){
     randomDate <- sample(possibleStarts, 1)
     blockStart <- max(randomDate,dayOne)
     blockEnd <- min(lastJulian,randomDate+blockLength-1)
-    oneYear <- subset(localSample, localSample$Julian >= blockStart & 
-                        localSample$Julian < blockEnd)
+    oneYear <- localSample[which(localSample$Julian >= blockStart & 
+                        localSample$Julian < blockEnd),]
     newSample <- rbind(oneYear, newSample)
         
   }
@@ -784,18 +800,7 @@ blockSample <- function(localSample, blockLength){
   return(newSample)
 }
 
-#' Assigns the narrative descriptors to the likelihood of change that is calculated by WBT
-#'
-#' Function called in \code{\link{wBT}} to convert numeric likelihood percentages to useful text.
-#'
-#' @param likeList list
-#' @return character vector for [1] Upward trend in concentration, 
-#' [2] Downward trend in concentration, [3] Upward trend in flux,
-#' [4] Downward trend in flux
-#' @export
-#' @examples
-#' likeList <- c(0.01, 0.5, 0.55, 0.99)
-#' Trends <- wordLike(likeList)
+
 wordLike <- function(likeList){
 	firstPart <- c("Upward trend in concentration is",
                  "Downward trend in concentration is",
@@ -819,13 +824,17 @@ wordLike <- function(likeList){
 
 #' pVal
 #'
-#' Computes the two-sided p value for the null hypothesis
+#' Computes the two-sided p value for the null hypothesis, where null 
+#' hypothesis is that the slope is zero, based on binomial distribution. 
+#' Should be noted that the result does not depend on the magnitude of the s 
+#' values only depends on the number of plus values and number of negative values.
 #'
-#' @param s slope values from the bootstrap (already flipped)
+#' @param s numeric vector of slope values from the bootstrap (already flipped)
 #' @export
+#' @return pVal numeric value
 #' @importFrom stats na.omit
 #' @examples
-#' s <- c(0.01, 0.5, 0.55, 0.99)
+#' s <- c(-1.0, 0, 0.5, 0.55, 3.0)
 #' pValue <- pVal(s)
 pVal <- function(s){
   # this function computes the two-sided p value for the null hypothesis
